#ifndef PARAMETERDUMP_H
#define PARAMETERDUMP_H

#include <QGroupBox>
#include <QGridLayout>
#include <QPushButton>
#include <QLabel>
#include <QSpinBox>
#include <QDoubleSpinBox>
#include <QCheckBox>
#include <QFile>
#include <QTextStream>

#include <tmpackettoread.h>
#include <TC_types.h>

#include "paulcommoncrc.h"
#include "tcpackettosend.h"

class ParameterDump  : public QWidget
{
    Q_OBJECT
public:
    explicit ParameterDump(QWidget *parent = 0);
    ~ParameterDump();

    void buildTabParameterDump();
    void buildCOMM();
    void buildNORM();
    void buildBURST();
    void buildSBM1();
    void buildSBM2();
    void buildDUMP();
    void updateParameterDump(TMPacketToRead *tmPacketToRead);
    void actionsSetEnabled(bool state);
    // k coefficients
    QString buildPrefix();
    void buildKCoefficientsFileName();
    void buildFBinsMasksFileName();
    void chooseDir();
    void processKCoefficientsDump(TMPacketToRead *packet );
    void processTMKCoefficientsDump( TMPacketToRead *packet );
    void processFBinsMasksDump( TMPacketToRead *packet );
    void readSettings();
    void writeSettings();

    QGroupBox* groupbox_dump;
    QGroupBox* groupbox_COMM;
    QGroupBox* groupbox_NORM;
    QGroupBox* groupbox_BURST;
    QGroupBox* groupbox_SBM1;
    QGroupBox* groupbox_SBM2;

    QGridLayout* layout_parameters;
    QGridLayout* layout_dump;
    QGridLayout* layout_COMM;
    QGridLayout* layout_NORM;
    QGridLayout* layout_BURST;
    QGridLayout* layout_SBM1;
    QGridLayout* layout_SBM2;

    QPushButton* button_loadCommon;
    QPushButton* button_loadNormal;
    QPushButton* button_loadBurst;
    QPushButton* button_loadSBM1;
    QPushButton* button_loadSBM2;
    QPushButton* button_parameterDump;
    QPushButton* button_kLoad;
    QPushButton* button_kDump;
    QPushButton* button_fBinsLoad;
    //
    QString defaultStorageDirectory;
    //
    QCheckBox *checkbox_kCoefficientsRecording;
    bool         kCoefficientsRecordFileEn;
    QFile       *kCoefficientsRecordFile;
    QTextStream *kCoefficientsRecordFileStrm;
    //
    QCheckBox *checkbox_fBinsMasksRecording;
    bool         fBinsMasksRecordFileEn;
    QFile       *fBinsMasksRecordFile;
    QTextStream *fBinsMasksRecordFileStrm;
    //
    QLabel* sy_lfr_bw;
    QLabel* sy_lfr_sp0;
    QLabel* sy_lfr_sp1;
    QLabel* sy_lfr_r0;
    QLabel* sy_lfr_r1;
    QLabel* sy_lfr_r2;
    //
    QLabel* sy_lfr_n_swf_l;
    QLabel* sy_lfr_n_swf_p;
    QLabel* sy_lfr_n_asm_p;
    QLabel* sy_lfr_n_bp_p0;
    QLabel* sy_lfr_n_bp_p1;
    QLabel* sy_lfr_n_cwf_long_f3;
    //
    QLabel* sy_lfr_b_bp_p0;
    QLabel* sy_lfr_b_bp_p1;
    //
    QLabel* sy_lfr_s1_bp_p0;
    QLabel* sy_lfr_s1_bp_p1;
    //
    QLabel* sy_lfr_s2_bp_p0;
    QLabel* sy_lfr_s2_bp_p1;

    QSpinBox* spinbox_sy_lfr_bw;
    QSpinBox* spinbox_sy_lfr_sp0;
    QSpinBox* spinbox_sy_lfr_sp1;
    QSpinBox* spinbox_sy_lfr_r0;
    QSpinBox* spinbox_sy_lfr_r1;
    QSpinBox* spinbox_sy_lfr_r2;
    //
    QSpinBox* spinbox_sy_lfr_n_swf_l;
    QSpinBox* spinbox_sy_lfr_n_swf_p;
    QSpinBox* spinbox_sy_lfr_n_asm_p;
    QSpinBox* spinbox_sy_lfr_n_bp_p0;
    QSpinBox* spinbox_sy_lfr_n_bp_p1;
    QSpinBox* spinbox_sy_lfr_n_cwf_long_f3;
    //
    QSpinBox* spinbox_sy_lfr_b_bp_p0;
    QSpinBox* spinbox_sy_lfr_b_bp_p1;
    //
    QSpinBox* spinbox_sy_lfr_s1_bp_p0;
    QSpinBox* spinbox_sy_lfr_s1_bp_p1;
    //
    QSpinBox* spinbox_sy_lfr_s2_bp_p0;
    QSpinBox* spinbox_sy_lfr_s2_bp_p1;

    PaulCommonCRC* paulCommonCRC;

signals:
    unsigned int WriteSPWSig(char *Value, unsigned int count, char targetLogicalAddress, char userApplication);

public slots:
    void sendParameterDump();
    void sendKDump();
    void storeKCoefficients( int state );
    void storeFBinsMasks( int state );
    void sendLoadCommon();
    void sendLoadNormal();
    void sendLoadBurst();
    void sendLoadSBM1();
    void sendLoadSBM2();

};

#endif // PARAMETERDUMP_H
