#ifndef ASMPAGE_H
#define ASMPAGE_H

#include <QMainWindow>
#include <QFile>
#include <QTextStream>
#include <QLabel>
#include <QRadioButton>
#include <QPushButton>
#include <QDockWidget>
#include <QToolBar>

#include "wfplot.h"
#include "spectrapacket.h"

class ASMPage : public QMainWindow
{
    Q_OBJECT
public:
    explicit ASMPage(QWidget *parent = 0, unsigned int bufferSize = 0, unsigned int xMAX = 0, unsigned int yMAX = 0);
    ~ASMPage();

    void createToolBar();
    void buildDockList();
    void unTabify();
    void tabify();
    // STORE
    QString buildPrefix();
    void buildFileName();

    bool logFileEn;
    bool storageEnabled;

    QLabel *logFileName;
    QLabel *label_storeWfrm;

    QWidget *titleWidgetB1B1;
    QWidget *titleWidgetB2B2;
    QWidget *titleWidgetB3B3;
    QWidget *titleWidgetE1E1;
    QWidget *titleWidgetE2E2;

    unsigned int localBufferSize;
    QByteArray **dataBuffer;

    QList<QDockWidget*> dockList;

    QDockWidget *dockB1B1;
    QDockWidget *dockB2B2;
    QDockWidget *dockB3B3;
    QDockWidget *dockE1E1;
    QDockWidget *dockE2E2;

    QRadioButton *radio_b1b1;
    QRadioButton *radio_b2b2;
    QRadioButton *radio_b3b3;
    QRadioButton *radio_e1e1;
    QRadioButton *radio_e2e2;
    QRadioButton *radio_tabify;

    QPushButton *button_selectAll;
    QPushButton *button_storeWfrm;

    QToolBar *myToolBar;

    WFPlot *wfPlot_asm_b1b1;
    WFPlot *wfPlot_asm_b2b2;
    WFPlot *wfPlot_asm_b3b3;
    WFPlot *wfPlot_asm_e1e1;
    WFPlot *wfPlot_asm_e2e2;

    QFile *logFile;

    QTextStream *logFileStrm;

    QString pageTitle;
    QString defaultStorageDirectory;

    void displayOnPlot(float *data, unsigned char num,
                       double deltaF, unsigned int asm_indice_start, unsigned int nbData);
    void initDataBuffer();
    void fillDataBuffer(float *data, unsigned char num, unsigned char asm_indice_start, double deltaF, unsigned int nbData);
    void storeSpectraPacket( SpectraPacket *spectraPacket, unsigned char asm_indice_start, double deltaF, unsigned int nbData);
    void storeDataBuffer(unsigned int nbData);

    bool allowDataStorage;

signals:

public slots:
    void actionRadioB1B1(bool state);
    void actionRadioB2B2(bool state);
    void actionRadioB3B3(bool state);
    void actionRadioE1E1(bool state);
    void actionRadioE2E2(bool state);
    void organizeDocks();
    void selectAll();
    void storeWfrm();
    void setDefaultStorageDirectory(QString nameOfTheDirectory);
    void updateScaleFactor(double scaleFactor);

};

#endif // ASMPAGE_H
